==> tuyaos_demo_ipc_1.0.0.tar.gz: tuyaos_demo_ipc/src/tkl/tkl_watchdog_core.c <==
/*
 * Copyright (C) 2021 Tuya Inc.
 */

#include <errno.h>
#include <sys/ioctl.h>
#include <fcntl.h>
#include <unistd.h>
#include <string.h>
#include <linux/watchdog.h>
#include "hal_types.h"
#include "hal_log.h"

#include "tkl_watchdog.h"
#include "tkl_watchdog_core.h"

static int wdog_fd = -1;


#define HAL_WATCHDOG_DEFAULT_DEVICE "/dev/watchdog"

static int hal_watchdog_open(void)
{
    char dev_name[MAX_PATH] = {0};
    int fd = -1;
    int ret;

    strncpy(dev_name, HAL_WATCHDOG_DEFAULT_DEVICE,
            strlen(HAL_WATCHDOG_DEFAULT_DEVICE) + 1);

    fd = open(dev_name, O_RDWR);
    if (fd < 0) {
        hal_log(HAL_LOG_ERR, "error: Failed to open %s\n", dev_name);
        return -errno;
    }

    return fd;
}

static int watchdog_ioctl(int fd, int ioctl_cmd, int* arg)
{
    int ret = 0;

    if (fd <= 0) {
        hal_log(HAL_LOG_ERR, "error: watchdog not initialized\n");
        return -EPERM;
    }

    ret = ioctl(fd, ioctl_cmd, arg);
    if (ret < 0) {
        hal_log(HAL_LOG_ERR, "error: ioctl 0x%X failed\n", ioctl_cmd);
        return -errno;
    }

    return ret;
}

static int hal_watchdog_close(int fd)
{
    char magic = 'V';
    int ret = 0;
    int option = WDIOS_DISABLECARD;

    if (fd <= 0) {
        hal_log(HAL_LOG_ERR, "error: watchdog not initialized\n");
        return -EPERM;
    }

    /*
     * A magic character 'V' must be written before closing the watchdog device.
     * Otherwise, the watchdog will reset the system after closing fd.
     */
    ret = write(fd, &magic, sizeof(magic));
    if (ret < 0) {
        hal_log(HAL_LOG_ERR, "error: write magic word failed\n");
    }

    ret = ioctl(fd, WDIOC_SETOPTIONS, &option);
    if(ret < 0) {
        hal_log(HAL_LOG_ERR, "error: WDIOC_SETOPTIONS failed\n");
    }

    close(fd);
    return ret;
}

static int hal_watchdog_keepalive(int fd)
{
    return watchdog_ioctl(fd, WDIOC_KEEPALIVE, NULL);
}

static int hal_watchdog_set_timeout(int fd, int* timeout)
{
    if (!timeout) {
        hal_log(HAL_LOG_ERR, "%s: Invaild argument\n", __func__);
        return -EINVAL;
    }

    return watchdog_ioctl(fd, WDIOC_SETTIMEOUT, timeout);
}

static int hal_watchdog_get_timeout(int fd, int* timeout)
{
    if (!timeout) {
        hal_log(HAL_LOG_ERR, "%s: Invaild argument\n", __func__);
        return -EINVAL;
    }

    return watchdog_ioctl(fd, WDIOC_GETTIMEOUT, timeout);
}

static int hal_watchdog_get_timeleft(int fd, int* timeleft)
{
    if (!timeleft) {
        hal_log(HAL_LOG_ERR, "%s: Invaild argument\n", __func__);
        return -EINVAL;
    }

    return watchdog_ioctl(fd, WDIOC_GETTIMELEFT, timeleft);
}

UINT_T tkl_watchdog_init_common(TKL_WDOG_CFG_T *cfg)
{
    int fd;
    int timeout; // in seconds

    if (cfg == NULL) {
        return 0;
    }

    fd = hal_watchdog_open();
    if (fd < 0) {
        return 0;
    }
    wdog_fd = fd;

    timeout = (int)cfg->interval_ms / 1000;
    if (hal_watchdog_set_timeout(wdog_fd, &timeout) != 0) {
        hal_log(HAL_LOG_ERR, "Failed to set watchdog timeout\n");
        // do not return; because the new timeout maybe invalid.
    }
    // get the effective timeout value
    if (hal_watchdog_get_timeout(wdog_fd, &timeout) != 0) {
        hal_log(HAL_LOG_ERR, "Failed to get watchdog timeout\n");
        return 0;
    }

    return (UINT_T)(timeout * 1000);
}

OPERATE_RET tkl_watchdog_deinit_common(VOID_T)
{
    if (wdog_fd != -1) {
        hal_watchdog_close(wdog_fd);
        wdog_fd = -1;
    }
    return OPRT_OK;
}

OPERATE_RET tkl_watchdog_refresh_common(VOID_T)
{
    if (hal_watchdog_keepalive(wdog_fd) != 0) {
        return OPRT_COM_ERROR;
    }
    return OPRT_OK;
}

==> tuyaos_demo_ipc_1.0.4.tar.gz: tuyaos_demo_ipc/src/tkl/tkl_watchdog_core.c <==
/*
 * Copyright (C) 2021 Tuya Inc.
 */

#include <errno.h>
#include <sys/ioctl.h>
#include <fcntl.h>
#include <unistd.h>
#include <string.h>
#include <linux/watchdog.h>
#include "hal_types.h"
#include "hal_log.h"

#include "tkl_watchdog.h"
#include "tkl_watchdog_core.h"

static int wdog_fd = -1;


#define HAL_WATCHDOG_DEFAULT_DEVICE "/dev/watchdog"

static int hal_watchdog_open(void)
{
    char dev_name[MAX_PATH] = {0};
    int fd = -1;
    int ret;

    strncpy(dev_name, HAL_WATCHDOG_DEFAULT_DEVICE,
            strlen(HAL_WATCHDOG_DEFAULT_DEVICE) + 1);

    fd = open(dev_name, O_RDWR);
    if (fd < 0) {
        hal_log(HAL_LOG_ERR, "error: Failed to open %s\n", dev_name);
        return -errno;
    }

    return fd;
}

static int watchdog_ioctl(int fd, int ioctl_cmd, int* arg)
{
    int ret = 0;

    if (fd <= 0) {
        hal_log(HAL_LOG_ERR, "error: watchdog not initialized\n");
        return -EPERM;
    }

    ret = ioctl(fd, ioctl_cmd, arg);
    if (ret < 0) {
        hal_log(HAL_LOG_ERR, "error: ioctl 0x%X failed\n", ioctl_cmd);
        return -errno;
    }

    return ret;
}

static int hal_watchdog_close(int fd)
{
    char magic = 'V';
    int ret = 0;
    int option = WDIOS_DISABLECARD;

    if (fd <= 0) {
        hal_log(HAL_LOG_ERR, "error: watchdog not initialized\n");
        return -EPERM;
    }

    /*
     * A magic character 'V' must be written before closing the watchdog device.
     * Otherwise, the watchdog will reset the system after closing fd.
     */
    ret = write(fd, &magic, sizeof(magic));
    if (ret < 0) {
        hal_log(HAL_LOG_ERR, "error: write magic word failed\n");
    }

    ret = ioctl(fd, WDIOC_SETOPTIONS, &option);
    if(ret < 0) {
        hal_log(HAL_LOG_ERR, "error: WDIOC_SETOPTIONS failed\n");
    }

    close(fd);
    return ret;
}

static int hal_watchdog_keepalive(int fd)
{
    return watchdog_ioctl(fd, WDIOC_KEEPALIVE, NULL);
}

static int hal_watchdog_set_timeout(int fd, int* timeout)
{
    if (!timeout) {
        hal_log(HAL_LOG_ERR, "%s: Invaild argument\n", __func__);
        return -EINVAL;
    }

    return watchdog_ioctl(fd, WDIOC_SETTIMEOUT, timeout);
}

static int hal_watchdog_get_timeout(int fd, int* timeout)
{
    if (!timeout) {
        hal_log(HAL_LOG_ERR, "%s: Invaild argument\n", __func__);
        return -EINVAL;
    }

    return watchdog_ioctl(fd, WDIOC_GETTIMEOUT, timeout);
}

static int hal_watchdog_get_timeleft(int fd, int* timeleft)
{
    if (!timeleft) {
        hal_log(HAL_LOG_ERR, "%s: Invaild argument\n", __func__);
        return -EINVAL;
    }

    return watchdog_ioctl(fd, WDIOC_GETTIMELEFT, timeleft);
}

UINT_T tkl_watchdog_init_common(TUYA_WDOG_BASE_CFG_T *cfg)
{
    int fd;
    int timeout; // in seconds

    if (cfg == NULL) {
        return 0;
    }

    fd = hal_watchdog_open();
    if (fd < 0) {
        return 0;
    }
    wdog_fd = fd;

    timeout = (int)cfg->interval_ms / 1000;
    if (hal_watchdog_set_timeout(wdog_fd, &timeout) != 0) {
        hal_log(HAL_LOG_ERR, "Failed to set watchdog timeout\n");
        // do not return; because the new timeout maybe invalid.
    }
    // get the effective timeout value
    if (hal_watchdog_get_timeout(wdog_fd, &timeout) != 0) {
        hal_log(HAL_LOG_ERR, "Failed to get watchdog timeout\n");
        return 0;
    }

    return (UINT_T)(timeout * 1000);
}

OPERATE_RET tkl_watchdog_deinit_common(VOID_T)
{
    if (wdog_fd != -1) {
        hal_watchdog_close(wdog_fd);
        wdog_fd = -1;
    }
    return OPRT_OK;
}

OPERATE_RET tkl_watchdog_refresh_common(VOID_T)
{
    if (hal_watchdog_keepalive(wdog_fd) != 0) {
        return OPRT_COM_ERROR;
    }
    return OPRT_OK;
}
